@@ -1,17 +1,17 @@
 /*
-    ccid.c: CCID common code
-    Copyright (C) 2003-2010   Ludovic Rousseau
-    Copyright (C) 2009-2018   Advanced Card Systems Ltd.
+	ccid.c: CCID common code
+	Copyright (C) 2003-2010   Ludovic Rousseau
+	Copyright (C) 2009-2023   Advanced Card Systems Ltd.
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either
-    version 2.1 of the License, or (at your option) any later version.
+	This library is free software; you can redistribute it and/or
+	modify it under the terms of the GNU Lesser General Public
+	License as published by the Free Software Foundation; either
+	version 2.1 of the License, or (at your option) any later version.
 
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
+	This library is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+	Lesser General Public License for more details.
 
 	You should have received a copy of the GNU Lesser General Public License
 	along with this library; if not, write to the Free Software Foundation,
@@ -88,6 +88,9 @@ int ccid_open_hack_pre(unsigned int reader_index)
 			break;
 
 		case SCM_SCL011:
+		case IDENTIV_uTrust3700F:
+		case IDENTIV_uTrust3701F:
+		case IDENTIV_uTrust4701F:
 			/* The SCM SCL011 reader needs 350 ms to answer */
 			ccid_descriptor->readTimeout = DEFAULT_COM_READ_TIMEOUT * 4;
 			break;
@@ -131,7 +134,7 @@ int ccid_open_hack_pre(unsigned int reader_index)
 
 		DEBUG_COMM("ICCD type A");
 		(void)CmdPowerOff(reader_index);
-		(void)CmdPowerOn(reader_index, &n, tmp, CCID_CLASS_AUTO_VOLTAGE);
+		(void)CmdPowerOn(reader_index, &n, tmp, VOLTAGE_AUTO);
 		(void)CmdPowerOff(reader_index);
 	}
 
@@ -151,7 +154,7 @@ int ccid_open_hack_pre(unsigned int reader_index)
 		}
 
 		(void)CmdPowerOff(reader_index);
-		(void)CmdPowerOn(reader_index, &n, tmp, CCID_CLASS_AUTO_VOLTAGE);
+		(void)CmdPowerOn(reader_index, &n, tmp, VOLTAGE_AUTO);
 		(void)CmdPowerOff(reader_index);
 	}
 
@@ -235,7 +238,7 @@ static void set_gemalto_firmware_features(unsigned int reader_index)
 		ret = CmdEscapeCheck(reader_index, cmd, sizeof cmd,
 			(unsigned char*)gf_features, &len_features, 0, TRUE);
 		if ((IFD_SUCCESS == ret) &&
-		    (len_features == sizeof *gf_features))
+			(len_features == sizeof *gf_features))
 		{
 			/* Command is supported if it succeeds at CCID level */
 			/* and returned size matches our expectation */
@@ -490,6 +493,7 @@ int ccid_open_hack_post(unsigned int reader_index)
 		case HPSMARTCARDKEYBOARD:
 		case HP_CCIDSMARTCARDKEYBOARD:
 		case FUJITSUSMARTKEYB:
+		case CHICONYHPSKYLABKEYBOARD:
 			/* the Secure Pin Entry is bogus so disable it
 			 * https://web.archive.org/web/20120320001756/http://martinpaljak.net/2011/03/19/insecure-hp-usb-smart-card-keyboard/
 			 *
@@ -513,6 +517,12 @@ int ccid_open_hack_post(unsigned int reader_index)
 			ccid_descriptor->dwFeatures |= CCID_CLASS_EXTENDED_APDU;
 			break;
 
+		case KOBIL_TRIBANK:
+			/* Firmware does NOT supported extended APDU */
+			ccid_descriptor->dwFeatures &= ~CCID_CLASS_EXCHANGE_MASK;
+			ccid_descriptor->dwFeatures |= CCID_CLASS_SHORT_APDU;
+			break;
+
 #if 0
 		/* SCM SCR331-DI contactless */
 		case SCR331DI:
@@ -573,6 +583,12 @@ int ccid_open_hack_post(unsigned int reader_index)
 			ccid_descriptor->readTimeout = DEFAULT_COM_READ_TIMEOUT;
 			break;
 
+		case BIT4ID_MINILECTOR:
+			/* The firmware 1.11 advertises pinpad but actually doesn't
+			 * have one */
+			ccid_descriptor->bPINSupport = 0;
+			break;
+
 		case ACS_ACR33U_A1_3SAM_ICC_READER:
 		case ACS_ACR33U_A2_3SAM_ICC_READER:
 		case ACS_ACR33U_A3_3SAM_ICC_READER:
@@ -628,6 +644,12 @@ int ccid_open_hack_post(unsigned int reader_index)
 						ccid_descriptor->dwMaxCCIDMessageLength = 320;
 					}
 				}
+
+				/*
+				 * Fix APG8201 which cannot receive the command properly.
+				 * Set write delay to 10 ms.
+				 */
+				ccid_descriptor->writeDelay = 10;
 			}
 		case ACS_APG8201Z:
 			ccid_descriptor->wLcdLayout = 0x0210;
@@ -681,6 +703,12 @@ int ccid_open_hack_post(unsigned int reader_index)
 					ccid_descriptor->firmwareFixEnabled = (firmwareVersion == 0x0001);
 					DEBUG_INFO2("Firmware fix enabled: %d", ccid_descriptor->firmwareFixEnabled);
 				}
+
+				/*
+				 * Fix ACR85 ICC which cannot receive the command properly.
+				 * Set write delay to 10 ms.
+				 */
+				ccid_descriptor->writeDelay = 10;
 			}
 			break;
 
@@ -817,34 +845,49 @@ int ccid_open_hack_post(unsigned int reader_index)
 
 		case ACS_ACR1251_1S_DUAL_READER:
 		case ACS_ACR1261_1S_DUAL_READER:
-		case ACS_ACR1281_1S_DUAL_READER:
 			// Adjust features and maximum data rate
 			if (ccid_descriptor->bCurrentSlotIndex == 0)
 			{
 				ccid_descriptor->dwFeatures = 0x000204BA;	// MCU
 				ccid_descriptor->dwMaxDataRate = 344100;
+			}
+			else if (ccid_descriptor->bCurrentSlotIndex == 1)
+			{
+				ccid_descriptor->dwFeatures = 0x0004047A;	// Contactless
+				ccid_descriptor->dwMaxDataRate = 344100;
+			}
+			else
+			{
+				ccid_descriptor->dwFeatures = 0x000204BA;	// SAM
+				ccid_descriptor->dwMaxDataRate = 125000;
+				ccid_descriptor->isSamSlot = 1;
+			}
+			break;
+
+		case ACS_ACR1281_1S_DUAL_READER:
+			// Adjust features and maximum data rate
+			if (ccid_descriptor->bCurrentSlotIndex == 0)
+			{
+				char firmwareVersion[30];
+				unsigned int firmwareVersionLen = sizeof(firmwareVersion);
+
+				ccid_descriptor->dwFeatures = 0x000204BA;	// MCU
+				ccid_descriptor->dwMaxDataRate = 344100;
 
 				/* ACR1281U-C1 >= v526 supports ICC extended APDU. */
-				if (ccid_descriptor->readerID == ACS_ACR1281_1S_DUAL_READER)
+				DEBUG_INFO1("Getting ACR1281U-C1 firmware version...");
+				if (ACR1222_GetFirmwareVersion(reader_index, firmwareVersion,
+					&firmwareVersionLen))
 				{
-					char firmwareVersion[30];
-					unsigned int firmwareVersionLen = sizeof(firmwareVersion);
+					int version = 0;
 
-					DEBUG_INFO1("Getting ACR1281U-C1 firmware version...");
-					if (ACR1222_GetFirmwareVersion(reader_index,
-						firmwareVersion, &firmwareVersionLen))
+					DEBUG_INFO2("ACR1281U-C1 firmware version: %s",
+						firmwareVersion);
+					if (sscanf(firmwareVersion, "ACR1281U_V%d", &version) > 0)
 					{
-						int version = 0;
-
-						DEBUG_INFO2("ACR1281U-C1 firmware version: %s",
-							firmwareVersion);
-						if (sscanf(firmwareVersion, "ACR1281U_V%d", &version)
-							> 0)
+						if (version >= 526)
 						{
-							if (version >= 526)
-							{
-								ccid_descriptor->dwFeatures = 0x000404BA;
-							}
+							ccid_descriptor->dwFeatures = 0x000404BA;
 						}
 					}
 				}
@@ -857,7 +900,7 @@ int ccid_open_hack_post(unsigned int reader_index)
 			else
 			{
 				ccid_descriptor->dwFeatures = 0x000204BA;	// SAM
-				ccid_descriptor->dwMaxDataRate = 125000;
+				ccid_descriptor->dwMaxDataRate = 688172;
 				ccid_descriptor->isSamSlot = 1;
 			}
 			break;
@@ -913,6 +956,10 @@ int ccid_open_hack_post(unsigned int reader_index)
 			}
 			break;
 
+		case ACS_ACR40T_ICC_READER:
+			ccid_descriptor->isSamSlot = 1;
+			break;
+
 		default:
 			break;
 	}
