@@ -303,6 +303,7 @@ typedef struct
 #define ACS_ACR1252_1S_CL_READER		0x072f223b
 #define ACS_ACR1252IMP_1S_CL_READER		0x072f2259
 #define ACS_WALLETMATE_1S_CL_READER		0x072f226b
+#define ACS_WALLETMATE_II_1S_CL_READER	0x072f2307
 #define ACS_ACR1261_1S_DUAL_READER		0x072f2211
 #define ACS_ACR128U						0x072f2100
 #define ACS_ACR1281_DUAL_READER_QPBOC	0x072f2208
@@ -318,6 +319,7 @@ typedef struct
 #define ACS_ACR1283_CL_READER			0x072f222C
 #define ACS_ACR1283U_FW_UPGRADE			0x072f220C
 #define ACS_ACR1552_1S_CL_READER		0x072f2303
+#define ACS_ACR1555_1S_CL_READER		0x072f230a
 #define ACS_ACR1581_1S_DUAL_READER		0x072f2301
 #define ACS_AET62_PICC_READER			0x072f0102
 #define ACS_AET62_1SAM_PICC_READER		0x072f0103
